@@ -3,13 +3,16 @@ package POE::Wheel::SocketFactory;
 use strict;
 
 use vars qw($VERSION);
-$VERSION = '1.292'; # NOTE - Should be #.### (three decimal places)
+$VERSION = '1.293'; # NOTE - Should be #.### (three decimal places)
 
 use Carp qw( carp croak );
 use Symbol qw( gensym );
 
 use Fcntl qw(F_GETFL F_SETFL O_NONBLOCK);
-use Errno qw(EWOULDBLOCK EADDRNOTAVAIL EINPROGRESS EADDRINUSE);
+use Errno qw(
+  EWOULDBLOCK EADDRNOTAVAIL EINPROGRESS EADDRINUSE ECONNABORTED
+  ESPIPE
+);
 use Socket qw(
   AF_INET SOCK_STREAM SOL_SOCKET AF_UNIX PF_UNIX 
   PF_INET SOCK_DGRAM SO_ERROR unpack_sockaddr_in 
@@ -182,7 +185,8 @@ sub _define_accept_state {
           $unique_id
         );
       }
-      elsif ($! != EWOULDBLOCK) {
+      elsif ($! != EWOULDBLOCK and $! != ECONNABORTED and $! != ESPIPE) {
+        # OSX reports ESPIPE, which isn't documented anywhere.
         $$event_failure && $k->call(
           $me, $$event_failure,
           'accept', ($!+0), $!, $unique_id
